import 'dart:async';
import 'dart:convert';
import 'dart:io';

import 'package:args/command_runner.dart';

import '../domain/repositories/auth_repositoty.dart';
import '../domain/repositories/packages_repository.dart';
import '../utils/get_config.dart';
import '../utils/loader.dart';

class PullCommand extends Command<void> {
  final AuthRepository _authRepository;
  final PackagesRepository _packagesRepository;

  PullCommand(this._authRepository, this._packagesRepository);
  @override
  String get description =>
      'Pull the ttf file and update the icons.dart file with the latest package changes from icons.meedu.app';

  @override
  String get name => 'pull';

  @override
  Future<void> run() async {
    try {
      final session = await showLoader(_authRepository.session);
      if (session == null) {
        throw Exception('❌ Unathorized: You must 👉 run micons login 👈');
      }

      final configFile = File('micons.json');

      if (!await configFile.exists()) {
        throw Exception('❌ micons.json file not found');
      }

      final config = jsonDecode(
        await showLoader(
          configFile.readAsString(),
        ),
      );

      final package = await showLoader(
        _packagesRepository.getPackage(
          session,
          config['id'],
        ),
      );

      if (package == null) {
        throw Exception('❌ Your request could not be processed');
      }

      final bytes = await showLoader(
        _packagesRepository.downloadSelectedIconsFont(
          session,
          package,
        ),
      );

      if (bytes == null) {
        throw Exception('❌ Your request could not be processed');
      }

      File(config['ttfFile']).writeAsBytesSync(bytes);
      File(config['dartFile']).writeAsStringSync(getDarCode(package));

      print('✅ pull sucessful');
      exit(0);
    } catch (e) {
      print(e);
      exit(1);
    }
  }
}
